/* ucs-cell-renderer-text.c
 *
 * Modified by Ricky Jang <shyblue@ucsoft.net>
 * Date : 2005.11.11
 * Change the color of fouced column on selected row
 * 
 * Copyright (C) 2002  Andreas J. Guelzow <aguelzow@taliesin.ca>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ucs-cell-renderer-text.h"
#include <gtk/gtk.h>

static void ucs_cell_renderer_text_class_init    
(UcsCellRendererTextClass *cell_text_class);

static GtkCellRendererTextClass *parent_class = NULL;

static void ucs_gtk_tree_view_get_cursor( 
        GtkTreeView         *treeview, 
        GtkTreePath         **path, 
        GtkTreeViewColumn   **focus_column, 
        GtkCellRenderer     **focus_renderer)
{
    GtkTreePath         *p_treepath         = NULL;
    GtkTreeViewColumn   *p_treeviewcolumn   = NULL;
    GtkCellRenderer     *p_cellrenderer     = NULL;
    GList               *p_list             = NULL;

    gtk_tree_view_get_cursor (treeview, &p_treepath, &p_treeviewcolumn);

    if (p_treeviewcolumn)
    {
        p_list = gtk_tree_view_column_get_cell_renderers (p_treeviewcolumn);

        if (p_list && p_list->data) 
        {
            p_cellrenderer = p_list->data;
        } 
        else 
        {
            p_cellrenderer = NULL;
        }
    }
    if (path) 
    {
        *path = p_treepath;
    }
    if (focus_column) 
    {
        *focus_column = p_treeviewcolumn;
    }
    if (focus_renderer)
    {
        *focus_renderer = p_cellrenderer;
    }
    if (p_list)
    {
        g_list_free (p_list);
    }
}

GType ucs_cell_renderer_text_get_type (void)
{
    static GType cell_text_type = 0;

    if (!cell_text_type)
    {
        static const GTypeInfo cell_text_info =
        {
            sizeof (UcsCellRendererTextClass),
            NULL,		/* base_init */
            NULL,		/* base_finalize */
            (GClassInitFunc)ucs_cell_renderer_text_class_init,
            NULL,		/* class_finalize */
            NULL,		/* class_data */
            sizeof (UcsCellRendererText),
            0,              /* n_preallocs */
            (GInstanceInitFunc) NULL,
        };

        cell_text_type = g_type_register_static (GTK_TYPE_CELL_RENDERER_TEXT, "UcsCellRendererText", &cell_text_info, 0);
    }

    return cell_text_type;
}

static void ucs_cell_renderer_text_render (
        GtkCellRenderer     *cell,
        GdkWindow           *window,
        GtkWidget           *widget,
        GdkRectangle        *background_area,
        GdkRectangle        *cell_area,
        GdkRectangle        *expose_area,
        GtkCellRendererState flags)

{
    GtkCellRendererText *celltext = (GtkCellRendererText *) cell;
    GtkStateType state;
    GdkGC *gc = gdk_gc_new (window);
    GtkCellRenderer *p_cell;
    GtkTreePath     *p_treepath_select;

    ucs_gtk_tree_view_get_cursor( GTK_TREE_VIEW( widget ), &p_treepath_select, NULL, &p_cell );

    if ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED)
    {
        if (GTK_WIDGET_HAS_FOCUS (widget))
            state = GTK_STATE_SELECTED;
        else
            state = GTK_STATE_ACTIVE;
    }
    else
    {
        if (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE)
            state = GTK_STATE_INSENSITIVE;
        else
            state = GTK_STATE_NORMAL;
    }

    if ( ( state == GTK_STATE_SELECTED )  && ( p_cell == cell ) )
    {
        gdk_draw_rectangle (window, gc, TRUE,
                background_area->x,
                background_area->y,
                background_area->width - 1,
                background_area->height - 1);
    }

    if (celltext->background_set)
    {
        GdkColor color;


        color.red = celltext->background.red;
        color.green = celltext->background.green;
        color.blue = celltext->background.blue;

        gdk_gc_set_rgb_fg_color (gc, &color);

        gdk_draw_rectangle (window,
                gc,
                TRUE,
                background_area->x,
                background_area->y + cell->ypad,
                background_area->width,
                background_area->height - 2 * cell->ypad);
    }

    gdk_gc_set_rgb_fg_color (gc, &widget->style->bg[
            ((flags & GTK_CELL_RENDERER_SELECTED) == GTK_CELL_RENDERER_SELECTED) ?
            GTK_STATE_ACTIVE : GTK_STATE_INSENSITIVE]);

    gdk_draw_rectangle (window, gc, FALSE,
            background_area->x,
            background_area->y,
            background_area->width - 1,
            background_area->height - 1);

    g_object_unref (G_OBJECT (gc));

    GTK_CELL_RENDERER_CLASS (parent_class)->render (cell, window, widget, background_area,
            cell_area, expose_area, flags);
}

static void ucs_cell_renderer_text_class_init (UcsCellRendererTextClass *class)
{
    GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS  (class);
    GObjectClass *object_class = G_OBJECT_CLASS (class);

    parent_class = g_type_class_peek_parent (object_class);

    cell_class->render = ucs_cell_renderer_text_render;
}

GtkCellRenderer *ucs_cell_renderer_text_new (void)
{
    return GTK_CELL_RENDERER (g_object_new (UCS_TYPE_CELL_RENDERER_TEXT, NULL));
}
