#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ucs-utils.h"

#define UPPER	('A'-'a')
#define LOWER	('a'-'A')

void *mymemcpy(void *s1,const void *s2, size_t n)
{
	char *su1;
	const char *su2;
	for(su1=s1,su2=s2;0<n;++su1,++su2,--n)
		*su1=*su2;
	return (s1);
}

void *mymemmove(void *s1, const void *s2, size_t n)
{
	char *sc1;
	const char *sc2;

	sc1=s1;
	sc2=s2;

	if( sc2<sc1 && sc2+n)
		for(sc1 += n,sc2 += n; 0<n; --n)
			*--sc1= *--sc2;
	else for (; 0<n; --n) *sc1++ = *sc2++;
	return ( s1);
}

void *mymemset(void *s,const int c, size_t n)
{
	const unsigned char uc=c;
	unsigned char *su;

	for (su = s ; 0<n; ++su, --n)
		*su=uc;
	return(s);
}

void *mycalloc( size_t nelem, size_t size)
{
	const size_t n = nelem *size;
	char *p = malloc (n) ;
	if (p) mymemset(p,'\0',n);
	return (p);
}

char *str_del(char *string, char str)
{
	char tmp[strlen(string)];
	char *buf;
	unsigned int i,cnt=0;

	mymemset(tmp,0x00,strlen(string));
	for(i=0;i<strlen(string);i++)
	{
		if (string[i]!=str)
		{
			tmp[i-cnt]=string[i];
		}
		else
			cnt++;
	}

	buf=malloc(sizeof(char) * ( strlen(string) - cnt)+1);
	strncpy(buf,tmp,strlen(string)-cnt);
	buf[i-cnt]='\0';
	return (buf);
}

int str_find( char *string, char *ch)
{
	char *buffer = string;
	while (*buffer)
	{
		if (strncmp(buffer,ch,(size_t)(strlen(ch))) == 0)
			return (1);
		else
			++buffer;
	}

	return 0;
}

char *str_right(char *src,int x)
{
	char *tmp;

	tmp = malloc(sizeof(char) * x +1);

	strncpy(tmp,src+(strlen(src)-x),x);
	tmp[x]='\0';

	return (tmp);
}

char *str_left(char *src,int x)
{
	char *tmp;

	tmp=malloc(sizeof(char) * x+1);
	strncpy(tmp,src,x);

	return (tmp);
}

char *str_sub(char *src,int ns,int x)
{
	char *tmp;

	tmp=malloc(sizeof(char)* x+1);

	strncpy(tmp,src+ns-1,x);
	tmp[x]='\0';

	return (tmp);
}

char *str_up( char *src)
{
	char *tmp;
	unsigned int i;

	tmp=malloc(sizeof(char)*strlen(src));

	for(i=0;i<strlen(src);i++)
	{
		if(src[i]>='a' && src[i]<'z') tmp[i]=src[i]+UPPER;
		else tmp[i]=src[i];
	}

	return (tmp);
}

char *str_down(char *src)
{
	char *tmp;
	unsigned int i;

	tmp=malloc(sizeof(char)*strlen(src));

	for(i=0;i<strlen(src);i++)
	{
		if(src[i]>='A' && src[i]<'Z') tmp[i]=src[i]+LOWER;
		else tmp[i]=src[i];
	}

	return (tmp);
}

char *str_replace(char *string, char *key,const char *value)
{
	int size = strlen(string);
	int cnt1= strlen(key);
	int cnt2= strlen(value);
	int i;
	char *tmp=NULL;

	tmp=malloc(sizeof(char)*size);
	strcpy(tmp,string);
	for(i=size-cnt1;i>=0;i--)
	{
		if( strncmp(tmp+i,key,(size_t)cnt1) == 0)
		{
			tmp=realloc(tmp,sizeof(char)*( strlen(tmp)+(cnt2-cnt1) ));
			mymemmove(tmp+i+cnt2,tmp+i+cnt1,strlen(tmp)-(i+cnt1));
			mymemcpy(tmp+i,value,cnt2);
		}
	}
	return (tmp);
}

char *str_duplicate(char *src,const char *key,int x)
{
	int i;

	char *tmp;
	tmp=malloc(sizeof(char)*(strlen(src)+strlen(key)*x));
	strcpy(tmp,src);
	for(i=0;i<x;i++)
	{
		strcat(tmp,key);
	}

	return (tmp);
}

char *str_padr(char *string,int x,const char pad)
{
	char *tmp;
	int rep = x - strlen(string);
	char buf[rep];
	int i;

	tmp= malloc(sizeof(char)*x+1);
	strcpy(tmp,string);

	if (rep > 0)
	{
		for(i=0;i<rep;i++)
		{
			buf[i]=pad;
		}
		buf[i]='\0';
	}
	strcat(tmp,buf);
	tmp[x]='\0';
	return (tmp);
}

char *str_padl(char *string,int x,const char pad)
{
	char *tmp;
	int rep = x - strlen(string);
	char buf[rep];
	int i;

	tmp= malloc(sizeof(char)*x+1);
	if (rep > 0)
	{
		for(i=0;i<rep;i++)
		{
			buf[i]=pad;
		}
		buf[i]='\0';
	}
	strcpy(tmp,buf);
	strcat(tmp,string);
	tmp[x]='\0';
	return (tmp);
}

char *str_rtrim(char *src)
{
	char *tmp;
	int i;

	for(i=strlen(src)-1;i>=0;i--)
	{
		if(src[i] != ' ') break;
	}

	tmp=malloc(sizeof(char)*i+2);
	strncpy(tmp,src,i+1);
	tmp[i+2]='\0';
	return (tmp);
}

char *str_ltrim(char *src)
{
	char *tmp;
	unsigned int i;

	for(i=0;i<strlen(src);i++)
	{
		if(src[i] != ' ') break;
	}

	tmp=malloc(sizeof(char)*(strlen(src)-i)+1);
	mymemcpy(tmp,src+i,strlen(src)-i);
	tmp[strlen(src)-i+1]='\0';
	return (tmp);
}

char *str_alltrim(char *src)
{
	char *tmp;

	tmp = str_del(src,' ');

	return (tmp);
}

void str_qs(char **cdata,int left,int right)
{
	int i,j;
	char *x,*y;

	i=left;
	j=right;
	x=cdata[(left+right)/2];

	do {
		while(strcmp(cdata[i],x) < 0 && i<right) i++;
		while(strcmp(x,cdata[j]) < 0 && j>left) j--;

		if(i<=j) {
			y = cdata[i];
			cdata[i]=cdata[j];
			cdata[j] = y ;
			i++;
			j--;
		}
	}while(i<=j);

	if(left<j) str_qs(cdata,left,j);
	if(i<right) str_qs(cdata,i,right);
}

Array ainit(Array array)
{
	array.nCnt=0;
	array.cdata=NULL;

	return array;
}
Array asort(Array array)
{
	str_qs(array.cdata,0,array.nCnt-1);

	return array;
}

Array aadd(Array array,const char *string)
{
	array.cdata = (char **)realloc(array.cdata,sizeof(char*) * (array.nCnt + 1));
	array.cdata[array.nCnt] = malloc(sizeof(char)*(strlen(string)+1));
	strcpy(array.cdata[array.nCnt],string);
	array.cdata[array.nCnt][strlen(string)]='\0';

	array.nCnt ++;
	return array;
}

Array adel(Array array,char *string)
{
	int rc=0,i;
	if( (rc=ascan(array,string) > 0) )
	{
		if ( array.nCnt != rc )
		{
			for(i=rc;i<array.nCnt-1;i++)
			{
				array.cdata[i]=(char *)realloc(array.cdata[i],sizeof(char)*(strlen(array.cdata[i+1]+1)));
				strcpy(array.cdata[i],array.cdata[i+1]);
			}
		}

		array.cdata=(char **)realloc(array.cdata,sizeof(char *)*(array.nCnt-1));
		array.nCnt--;
	}

	return array;
}

Array aclone(Array dest, Array src)
{
	int i;
	dest.nCnt=0;
	dest.cdata=NULL;

	dest.cdata=(char **)realloc(dest.cdata,sizeof(char*) * (src.nCnt+1));
	for (i=0;i<src.nCnt;i++)
	{
		dest.cdata[i] = malloc(sizeof(char)*(strlen(src.cdata[i])+1));
		strcpy(dest.cdata[i],src.cdata[i]);
	}

	dest.nCnt = src.nCnt;
	return dest;
}

int ascan(Array array, char *key)
{
	int i;

	for(i=0;i<array.nCnt;i++)
	{
		if(strcmp(array.cdata[i],key) == 0)
			return i;
	}
	return -1;
}

Tbl *tblinit(Tbl *tbl)
{
	tbl = malloc(sizeof(Tbl));
	tbl->nCnt=0;
	tbl->array=NULL;

	return tbl;
}
Tbl *tbladd(Tbl *tbl,Array array)
{
	tbl->array = (Array *)realloc(tbl->array,sizeof(Array)*(tbl->nCnt+1));
	tbl->array[tbl->nCnt]=aclone(tbl->array[tbl->nCnt],array);

	tbl->nCnt++;
	return tbl;
}
Tbl *tblremove(Tbl *tbl,int index)
{
    int i;
    
    if ( index < tbl->nCnt-1 )
    {
        for( i = index; i < tbl->nCnt-1; i++ )
        {
            tbl->array[i] = tbl->array[i+1];
        }
    }
    tbl->nCnt--;
    tbl->array = (Array *)realloc(tbl->array,sizeof(Array)*(tbl->nCnt));

    return tbl;
}

#ifdef DEV
int main(void)
{
	char *string = "this is my test";
	char *tmp=NULL;
	int itmp,i,j;
	Array array;
	Array arr2;
	Tbl *tbl=NULL; 

	tmp=str_left(string,4);
	printf("left 4 string of '%s' is %s \n",string,tmp);

	tmp=str_right(string,10);
	printf("right 10 string of '%s' is %s \n",string,tmp);

	tmp=str_sub(string,2,5);
	printf("sub string 5 at 2 of '%s' is %s \n",string,tmp);

	tmp=str_del(string,'i');
	printf("get rid of 'i' form '%s' is %s \n",string,tmp);

	tmp=str_up(string);
	printf("Upper %s is %s\n",string,tmp);

	string=str_down(tmp);
	printf("Lower %s is %s\n",tmp,string);

	tmp=str_replace(string,"is","sssiiisssiii");
	printf("Replace is to sssiiisssiii : %s\n",tmp);

	tmp=str_duplicate(string,"xx",5);
	printf("Duplicate xx 5time to %s : %s\n",string,tmp);

	tmp=str_padr("312",9,'0');
	printf("padr test %s\n",tmp);
	itmp = atoi(tmp);
	printf("%d \n",itmp);

	tmp=str_padl("312",9,'0');
	printf("padl test %s\n",tmp);

	itmp = atoi(tmp);
	printf("%d \n",itmp);

	printf("Original Data :12124513     |(| is end mark)\n");
	tmp=str_rtrim("12124513     ");
	printf("Rtrimed Data  :%s|\n",tmp);

	printf("Original data :    12124513\n");
	tmp=str_ltrim("    12124513");
	printf("Ltrimed data  :%s\n",tmp);

	printf("Original data :    1212 4 5 1 3   |(| is end mark)\n");
	tmp=str_alltrim("    1212 4 5 1 3   ");
	printf("alltrimed data:%s|\n",tmp);

	array=ainit(array);
	printf("aadd test \n");
	array=aadd(array,"test");
	array=aadd(array,"test32");
	array=aadd(array,"test2");
	array=aadd(array,"카카카");
	for(i=0;i<array.nCnt;i++)
	{
		printf("Array[%d] is %s \n",i,array.cdata[i]);
	}

	printf("asort test \n");
	array=asort(array);
	for(i=0;i<array.nCnt;i++)
	{
		printf("Array[%d] is %s \n",i,array.cdata[i]);
	}

	arr2=aclone(arr2,array);

	printf("adel test \n");
	array=adel(array,"test2");
	for(i=0;i<array.nCnt;i++)
	{
		printf("Array[%d] is %s \n",i,array.cdata[i]);
	}
	printf("aclone test \n");
	for(i=0;i<arr2.nCnt;i++)
	{
		printf("Arr2[%d] is %s \n",i,arr2.cdata[i]);
	}

	tbl=tblinit(tbl);
	tbl=tbladd(tbl,arr2);
	tbl=tbladd(tbl,array);
	tbl=tbladd(tbl,array);
	tbl=tbladd(tbl,arr2);
	array=aadd(array,"asdadgsdhF");
	array=aadd(array,"adgsdh");
	array=aadd(array,"dfghghety");
	array=aadd(array,"rtyjfgj");
	array=aadd(array,"rtyjfghktyketyket");
	array=aadd(array,"etjydvhgcvnrtj7y");
	tbl=tbladd(tbl,array);
	printf("Table add (tbladd) test\n");
    tbl=tblremove(tbl,0);
	for(i=0;i<tbl->nCnt;i++)
	{
		printf("%d ",i);
		for(j=0;j<tbl->array[i].nCnt;j++)
		{
			printf("%s\t",tbl->array[i].cdata[j]);
		}
		printf("\n");
	}
	return 0;
}
#endif
